@@ -104,6 +104,7 @@ typedef float               GLclampf;   /* single precision float in [0,1] */
 typedef double              GLdouble;   /* double precision float */
 typedef double              GLclampd;   /* double precision float in [0,1] */
 
+#define GL_NO_ERROR                 0x0000
 #define GL_VERSION                  0x1F02
 #define GL_UNSIGNED_BYTE            0x1401
 #define GL_UNSIGNED_INT             0x1405
@@ -112,6 +113,7 @@ typedef double              GLclampd;   /* double precision float in [0,1] */
 #define GL_RGBA                     0x1908
 #define GL_COLOR_BUFFER_BIT         0x00004000
 #define GL_CONTEXT_FLAGS            0x821e
+#define GL_CONTEXT_ROBUST_ACCESS    0x90F3
 
 #define GL_CONTEXT_FLAG_FORWARD_COMPATIBLE_BIT 0x00000001
 #define GL_CONTEXT_FLAG_DEBUG_BIT              0x00000002
@@ -260,6 +262,7 @@ gl_basic_fini(void **state)
         .profile = WAFFLE_DONT_CARE, \
         .forward_compatible = false, \
         .debug = false, \
+        .robust = false, \
         .alpha = false, \
         .expect_error = WAFFLE_NO_ERROR, \
         __VA_ARGS__ \
@@ -272,6 +275,7 @@ struct gl_basic_draw_args__ {
     int32_t expect_error;
     bool forward_compatible;
     bool debug;
+    bool robust;
     bool alpha;
 };
 
@@ -285,6 +289,7 @@ gl_basic_draw__(void **state, struct gl_basic_draw_args__ args)
     int32_t expect_error = args.expect_error;
     bool context_forward_compatible = args.forward_compatible;
     bool context_debug = args.debug;
+    bool context_robust = args.robust;
     bool alpha = args.alpha;
 
     int32_t config_attrib_list[64];
@@ -317,6 +322,10 @@ gl_basic_draw__(void **state, struct gl_basic_draw_args__ args)
         config_attrib_list[i++] = WAFFLE_CONTEXT_DEBUG;
         config_attrib_list[i++] = true;
     }
+    if (context_robust) {
+        config_attrib_list[i++] = WAFFLE_CONTEXT_ROBUST_ACCESS;
+        config_attrib_list[i++] = true;
+    }
     config_attrib_list[i++] = WAFFLE_RED_SIZE;
     config_attrib_list[i++] = 8;
     config_attrib_list[i++] = WAFFLE_GREEN_SIZE;
@@ -379,20 +388,69 @@ gl_basic_draw__(void **state, struct gl_basic_draw_args__ args)
     assert_true(waffle_get_current_window() == ts->window);
     assert_true(waffle_get_current_context() == ts->ctx);
 
-    const char *version_str;
+    const char *version_str, *expected_version_str;
     int major, minor, count;
 
     ASSERT_GL(version_str = (const char *) glGetString(GL_VERSION));
     assert_true(version_str != NULL);
 
-    while (*version_str != '\0' && !isdigit(*version_str))
-        version_str++;
+    switch (waffle_context_api) {
+    case WAFFLE_CONTEXT_OPENGL:
+	expected_version_str = "";
+        break;
+    case WAFFLE_CONTEXT_OPENGL_ES1:
+	expected_version_str = "OpenGL ES-CM ";
+        break;
+    case WAFFLE_CONTEXT_OPENGL_ES2:
+    case WAFFLE_CONTEXT_OPENGL_ES3:
+	expected_version_str = "OpenGL ES ";
+        break;
+    default:
+        assert_true(0);
+        break;
+    }
+
+    const size_t version_str_len = strlen(expected_version_str);
+    assert_true(strncmp(version_str, expected_version_str, version_str_len) == 0);
+    version_str += version_str_len;
 
     count = sscanf(version_str, "%d.%d", &major, &minor);
     assert_true(count == 2);
     assert_true(major >= 0);
     assert_true(minor >= 0 && minor < 10);
 
+    if (context_version != WAFFLE_DONT_CARE) {
+        int expected_major = context_version / 10;
+        int expected_minor = context_version % 10;
+
+        assert_true(major >= expected_major);
+        assert_true(minor >= expected_minor || major > expected_major);
+    }
+
+    const char *profile_suffix = "";
+
+    if (waffle_context_api == WAFFLE_CONTEXT_OPENGL) {
+        switch (context_profile) {
+        case WAFFLE_CONTEXT_CORE_PROFILE:
+            profile_suffix = " (Core Profile)";
+            break;
+        case WAFFLE_CONTEXT_COMPATIBILITY_PROFILE:
+            profile_suffix = " (Compatibility Profile)";
+            break;
+        case WAFFLE_DONT_CARE:
+            break;
+        default:
+            assert_true(0);
+            break;
+        }
+    }
+
+    char profile_str[30]; // 30 should be enough ;-)
+    sprintf(profile_str, "%d.%d%s", major, minor, profile_suffix);
+
+    const size_t profile_str_len = strlen(profile_str);
+    assert_true(strncmp(version_str, profile_str, profile_str_len) == 0);
+
     int version_10x = 10 * major + minor;
 
     if ((waffle_context_api == WAFFLE_CONTEXT_OPENGL && version_10x >= 30) ||
@@ -411,6 +469,21 @@ gl_basic_draw__(void **state, struct gl_basic_draw_args__ args)
         }
     }
 
+    // GL_ROBUST_ACCESS comes with the following extensions
+    // GL_ARB_robustness
+    // GL_EXT_robustness
+    // GL_KHR_robustness
+    //
+    // To keep it simple, assume the correct extension is there if
+    // glGetError is happy ;-)
+    if (context_robust) {
+        GLint robust_flag = 0;
+        glGetIntegerv(GL_CONTEXT_ROBUST_ACCESS, &robust_flag);
+
+        if (glGetError() == GL_NO_ERROR)
+            assert_true(robust_flag);
+    }
+
     // Draw.
     ASSERT_GL(glClearColor(RED_F, GREEN_F, BLUE_F, ALPHA_F));
     ASSERT_GL(glClear(GL_COLOR_BUFFER_BIT));
@@ -453,15 +526,24 @@ static void test_gl_basic_##context_api##_fwdcompat(void **state)       \
                   .expect_error=WAFFLE_##error);                        \
 }
 
-#define test_gl_debug(error)                                            \
-static void test_gl_basic_gl_debug(void **state)                        \
+#define test_XX_debug(context_api, waffle_api, error)                   \
+static void test_gl_basic_##context_api##_debug(void **state)           \
 {                                                                       \
     gl_basic_draw(state,                                                \
-                  .api=WAFFLE_CONTEXT_OPENGL,                           \
+                  .api=WAFFLE_CONTEXT_##waffle_api,                     \
                   .debug=true,                                          \
                   .expect_error=WAFFLE_##error);                        \
 }
 
+#define test_XX_robust(context_api, waffle_api, error)                  \
+static void test_gl_basic_##context_api##_robust(void **state)          \
+{                                                                       \
+    gl_basic_draw(state,                                                \
+                  .api=WAFFLE_CONTEXT_##waffle_api,                     \
+                  .robust=true,                                         \
+                  .expect_error=WAFFLE_##error);                        \
+}
+
 #define test_glXX(waffle_version, error)                                \
 static void test_gl_basic_gl##waffle_version(void **state)              \
 {                                                                       \
@@ -502,6 +584,28 @@ static void test_gl_basic_gl##waffle_version##_core_fwdcompat(void **state) \
                   .expect_error=WAFFLE_##error);                        \
 }
 
+#define test_glXX_core_debug(waffle_version, error)                     \
+static void test_gl_basic_gl##waffle_version##_core_debug(void **state) \
+{                                                                       \
+    gl_basic_draw(state,                                                \
+                  .api=WAFFLE_CONTEXT_OPENGL,                           \
+                  .version=waffle_version,                              \
+                  .profile=WAFFLE_CONTEXT_CORE_PROFILE,                 \
+                  .debug=true,                                          \
+                  .expect_error=WAFFLE_##error);                        \
+}
+
+#define test_glXX_core_robust(waffle_version, error)                    \
+static void test_gl_basic_gl##waffle_version##_core_robust(void **state) \
+{                                                                       \
+    gl_basic_draw(state,                                                \
+                  .api=WAFFLE_CONTEXT_OPENGL,                           \
+                  .version=waffle_version,                              \
+                  .profile=WAFFLE_CONTEXT_CORE_PROFILE,                 \
+                  .robust=true,                                         \
+                  .expect_error=WAFFLE_##error);                        \
+}
+
 #define test_glXX_compat(waffle_version, error)                         \
 static void test_gl_basic_gl##waffle_version##_compat(void **state)     \
 {                                                                       \
@@ -512,6 +616,28 @@ static void test_gl_basic_gl##waffle_version##_compat(void **state)     \
                   .expect_error=WAFFLE_##error);                        \
 }
 
+#define test_glXX_compat_debug(waffle_version, error)                   \
+static void test_gl_basic_gl##waffle_version##_compat_debug(void **state) \
+{                                                                       \
+    gl_basic_draw(state,                                                \
+                  .api=WAFFLE_CONTEXT_OPENGL,                           \
+                  .version=waffle_version,                              \
+                  .profile=WAFFLE_CONTEXT_COMPATIBILITY_PROFILE,        \
+                  .debug=true,                                          \
+                  .expect_error=WAFFLE_##error);                        \
+}
+
+#define test_glXX_compat_robust(waffle_version, error)                  \
+static void test_gl_basic_gl##waffle_version##_compat_robust(void **state) \
+{                                                                       \
+    gl_basic_draw(state,                                                \
+                  .api=WAFFLE_CONTEXT_OPENGL,                           \
+                  .version=waffle_version,                              \
+                  .profile=WAFFLE_CONTEXT_COMPATIBILITY_PROFILE,        \
+                  .robust=true,                                         \
+                  .expect_error=WAFFLE_##error);                        \
+}
+
 #define test_glesXX(api_version, waffle_version, error)                 \
 static void test_gl_basic_gles##waffle_version(void **state)            \
 {                                                                       \
@@ -539,6 +665,7 @@ testsuite_##platform(void)                                              \
         unit_test_make(test_gl_basic_gl_rgba),                          \
         unit_test_make(test_gl_basic_gl_fwdcompat),                     \
         unit_test_make(test_gl_basic_gl_debug),                         \
+        unit_test_make(test_gl_basic_gl_robust),                        \
                                                                         \
         unit_test_make(test_gl_basic_gl10),                             \
         unit_test_make(test_gl_basic_gl11),                             \
@@ -557,6 +684,8 @@ testsuite_##platform(void)                                              \
                                                                         \
         unit_test_make(test_gl_basic_gl32_core),                        \
         unit_test_make(test_gl_basic_gl32_core_fwdcompat),              \
+        unit_test_make(test_gl_basic_gl32_core_debug),                  \
+        unit_test_make(test_gl_basic_gl32_core_robust),                 \
         unit_test_make(test_gl_basic_gl33_core),                        \
         unit_test_make(test_gl_basic_gl40_core),                        \
         unit_test_make(test_gl_basic_gl41_core),                        \
@@ -564,6 +693,8 @@ testsuite_##platform(void)                                              \
         unit_test_make(test_gl_basic_gl43_core),                        \
                                                                         \
         unit_test_make(test_gl_basic_gl32_compat),                      \
+        unit_test_make(test_gl_basic_gl32_compat_debug),                \
+        unit_test_make(test_gl_basic_gl32_compat_robust),               \
         unit_test_make(test_gl_basic_gl33_compat),                      \
         unit_test_make(test_gl_basic_gl40_compat),                      \
         unit_test_make(test_gl_basic_gl41_compat),                      \
@@ -573,17 +704,23 @@ testsuite_##platform(void)                                              \
         unit_test_make(test_gl_basic_gles1_rgb),                        \
         unit_test_make(test_gl_basic_gles1_rgba),                       \
         unit_test_make(test_gl_basic_gles1_fwdcompat),                  \
+        unit_test_make(test_gl_basic_gles1_robust),                     \
+        unit_test_make(test_gl_basic_gles1_debug),                      \
         unit_test_make(test_gl_basic_gles10),                           \
         unit_test_make(test_gl_basic_gles11),                           \
                                                                         \
         unit_test_make(test_gl_basic_gles2_rgb),                        \
         unit_test_make(test_gl_basic_gles2_rgba),                       \
         unit_test_make(test_gl_basic_gles2_fwdcompat),                  \
+        unit_test_make(test_gl_basic_gles2_debug),                      \
+        unit_test_make(test_gl_basic_gles2_robust),                     \
         unit_test_make(test_gl_basic_gles20),                           \
                                                                         \
         unit_test_make(test_gl_basic_gles3_rgb),                        \
         unit_test_make(test_gl_basic_gles3_rgba),                       \
         unit_test_make(test_gl_basic_gles3_fwdcompat),                  \
+        unit_test_make(test_gl_basic_gles3_debug),                      \
+        unit_test_make(test_gl_basic_gles3_robust),                     \
         unit_test_make(test_gl_basic_gles30),                           \
                                                                         \
     };                                                                  \
@@ -612,7 +749,8 @@ test_glXX_fwdcompat(21, ERROR_BAD_ATTRIBUTE)
 //
 
 test_XX_fwdcompat(gl, OPENGL, ERROR_BAD_ATTRIBUTE)
-test_gl_debug(NO_ERROR)
+test_XX_debug(gl, OPENGL, NO_ERROR)
+test_XX_robust(gl, OPENGL, NO_ERROR)
 
 test_glXX(30, NO_ERROR)
 test_glXX_fwdcompat(30, NO_ERROR)
@@ -621,6 +759,8 @@ test_glXX_fwdcompat(31, NO_ERROR)
 
 test_glXX_core(32, NO_ERROR)
 test_glXX_core_fwdcompat(32, NO_ERROR)
+test_glXX_core_debug(32, NO_ERROR)
+test_glXX_core_robust(32, NO_ERROR)
 test_glXX_core(33, NO_ERROR)
 test_glXX_core(40, NO_ERROR)
 test_glXX_core(41, NO_ERROR)
@@ -628,6 +768,8 @@ test_glXX_core(42, NO_ERROR)
 test_glXX_core(43, NO_ERROR)
 
 test_glXX_compat(32, NO_ERROR)
+test_glXX_compat_debug(32, NO_ERROR)
+test_glXX_compat_robust(32, NO_ERROR)
 test_glXX_compat(33, NO_ERROR)
 test_glXX_compat(40, NO_ERROR)
 test_glXX_compat(41, NO_ERROR)
@@ -636,15 +778,21 @@ test_glXX_compat(43, NO_ERROR)
 
 test_XX_rgb(gles1, OPENGL_ES1, NO_ERROR)
 test_XX_rgba(gles1, OPENGL_ES1, NO_ERROR)
+test_XX_debug(gles1, OPENGL_ES1, NO_ERROR)
+test_XX_robust(gles1, OPENGL_ES1, NO_ERROR)
 test_glesXX(1, 10, NO_ERROR)
 test_glesXX(1, 11, NO_ERROR)
 
 test_XX_rgb(gles2, OPENGL_ES2, NO_ERROR)
 test_XX_rgba(gles2, OPENGL_ES2, NO_ERROR)
+test_XX_debug(gles2, OPENGL_ES2, NO_ERROR)
+test_XX_robust(gles2, OPENGL_ES2, NO_ERROR)
 test_glesXX(2, 20, NO_ERROR)
 
 test_XX_rgb(gles3, OPENGL_ES3, NO_ERROR)
 test_XX_rgba(gles3, OPENGL_ES3, NO_ERROR)
+test_XX_debug(gles3, OPENGL_ES3, NO_ERROR)
+test_XX_robust(gles3, OPENGL_ES3, NO_ERROR)
 test_glesXX(3, 30, NO_ERROR)
 
 //
@@ -749,13 +897,18 @@ CREATE_TESTSUITE(WAFFLE_PLATFORM_WGL, wgl)
 
 #undef test_glesXX
 
+#undef test_glXX_compat_robust
+#undef test_glXX_compat_debug
 #undef test_glXX_compat
+#undef test_glXX_core_robust
+#undef test_glXX_core_debug
 #undef test_glXX_core_fwdcompat
 #undef test_glXX_core
 #undef test_glXX_fwdcompat
 #undef test_glXX
 
-#undef test_gl_debug
+#undef test_XX_robust
+#undef test_XX_debug
 #undef test_XX_fwdcompat
 #undef test_XX_rgba
 #undef test_XX_rgb
@@ -886,10 +1039,11 @@ enum_map_translate_str(
 
 /// @return true on success.
 static bool
-parse_args(int argc, char *argv[], int *platform)
+parse_args(int argc, char *argv[], int *out_platform)
 {
     bool ok;
     bool loop_get_opt = true;
+    int platform = 0;
 
 #ifdef __APPLE__
     removeXcodeArgs(&argc, argv);
@@ -907,7 +1061,7 @@ parse_args(int argc, char *argv[], int *platform)
             case '?':
                 goto error_unrecognized_arg;
             case OPT_PLATFORM:
-                ok = enum_map_translate_str(platform_map, optarg, platform);
+                ok = enum_map_translate_str(platform_map, optarg, &platform);
                 if (!ok) {
                     usage_error_printf("'%s' is not a valid platform",
                                        optarg);
@@ -926,10 +1080,11 @@ parse_args(int argc, char *argv[], int *platform)
         goto error_unrecognized_arg;
     }
 
-    if (!*platform) {
+    if (!platform) {
         usage_error_printf("--platform is required");
     }
 
+    *out_platform = platform;
     return true;
 
 error_unrecognized_arg:
